#!/bin/sh
# --------------------------------------------------------------------------------------
# Copyright (C) 2017 SOS Berlin GmbH, www.sos-berlin.com
# Author:  Andreas Pueschel, Oliver Haufe
# Mail:    andreas.pueschel@sos-berlin.com, oliver.haufe@sos-berlin.com
# Purpose: Submit event to JobScheduler Supervisor Instance
# --------------------------------------------------------------------------------------

test -z "$TMP" && TMP="/tmp"
TMPDIR="$TMP/${USER}"
test -d "$TMPDIR" || mkdir "$TMPDIR" >/dev/null
test -z "$SCHEDULER_DATA" && SCHEDULER_DATA="$TMPDIR"
now=`date -u "+%Y-%m-%d %H:%M:%S"`
thisname=`basename "$0"`
thisdir=`dirname "$0"`
test "${SCHEDULER_HOME}" != "" && thisdir="${SCHEDULER_HOME}/bin"
tmpprefix="${TMPDIR}/sos_${RANDOM}"
queued_events="${TMPDIR}/scheduler.events"

exit_code=
exit_code2=
joc_url="${SCHEDULER_JOC_URL}"
joc_account="${SCHEDULER_JOC_ACCOUNT}"
http_timeout="${SCHEDULER_HTTP_TIMEOUT}"
job_chain="${SCHEDULER_JOB_CHAIN}"
order_id="${SCHEDULER_ORDER_ID}"
job_name="${SCHEDULER_JOB_NAME}"
scheduler_id="${SCHEDULER_ID}"
scheduler_host=${SCHEDULER_HOST}
scheduler_tcp_port=${SCHEDULER_TCP_PORT}
scheduler_supervisor_host=${SCHEDULER_SUPERVISOR_HOST}
scheduler_supervisor_port=${SCHEDULER_SUPERVISOR_PORT}
event_job_chain=/sos/events/scheduler_event_service
what=add
expires=""
expire_param=""
expire_json_param=""
parameters=""
json_parameters=""
use_web_service=0
allowed_exit_codes=""
ACCESS_TOKEN=""



my_echo="echo -e"
test "`echo -e`" = "-e" && my_echo="echo"


usage () {
  $my_echo >&2 \
    "usage: $thisname \n" \
    "         -x exit-code \n" \
    "         -e event-class \n" \
    "         -i event-id,                      default: exit code \n" \
    "        [-l allowed-exit-code[,allowed-exit-code[,...]]] \n" \
    "        [-j job-name,                      default: \$SCHEDULER_JOB_NAME] \n" \
    "        [-c job-chain,                     default: \$SCHEDULER_JOB_CHAIN] \n" \
    "        [-o order-id,                      default: \$SCHEDULER_ORDER_ID] \n" \
    "        [-h workload-job-scheduler-host,   default: \$SCHEDULER_HOST] \n" \
    "        [-p workload-job-scheduler-port,   default: \$SCHEDULER_TCP_PORT] \n" \
    "        [-s supervisor-job-scheduler-host, default: \$SCHEDULER_SUPERVISOR_HOST] \n" \
    "        [-r supervisor-job-scheduler-port, default: \$SCHEDULER_SUPERVISOR_PORT] \n" \
    "        [-k supervisor-job-chain,          default: /sos/events/scheduler_event_service] \n" \
    "        [-w add|remove|check,              default: add] \n" \
    "        [-t never|expiration-date (ISO)] \n" \
    "        [-tc expiration-cycle (hh:mm[:ss])] \n" \
    "        [-tp expiration-period (hh:mm[:ss])] \n" \
    "        [-d name=value ...-d name2=value2] \n" \
    "            if 'value' contains special chars like &<>|'!;\" \n" \
    "            wrap the 'value' with double quotes (\") \n" \
    "            and type \\! and \\\" instead of ! and \" \n" \
    "        [-a XPath expression (for -w check)] \n" \
    "        [-u JOC-url                        default: \$SCHEDULER_JOC_URL \n" \
    "            or joc.url from \$SCHEDULER_DATA/private/private.conf] \n" \
    "        [-n JOC-account:JOC-password]      default: \$SCHEDULER_JOC_ACCOUNT \n" \
    "            or joc.account from \$SCHEDULER_DATA/private/private.conf] \n" \
    "        [-b JobScheduler id]               default: \$SCHEDULER_ID] \n" \
    "        [-m HTTP timeout in ms]            default: \$SCHEDULER_HTTP_TIMEOUT \n" \
    "            or joc.timeout from \$SCHEDULER_DATA/private/private.conf or 15000] \n"
  exit 1
}

check_arg () {
  if [ "$1" = "" ]; then
    $my_echo >&2 "The option $2 requires an argument"
    usage
  fi
  tmp_arg=`echo "$1" | cut -c 1`
  if [ "$tmp_arg" = "-" ]; then
    $my_echo >&2 "The option $2 requires an argument"
    usage
  fi
  if [ "$3" != "expression" ]; then
    tmp_arg=`$my_echo "$1" | sed -e 's/[<>"\|!;]//g' -e "s/'//g"`
    if [ "$tmp_arg" != "$1" ]; then
      if [ "$3" = "number" ]; then
        $my_echo >&2 "The value of $2 must be a number: $1"
      else
        $my_echo >&2 "The value of $2 may not contain any special chars like <>|'!;\": $1"
      fi
      usage
    else
      if [ "$3" = "number" ]; then
        tmp_arg=`$my_echo "$1" | sed -e 's/[^0-9]//g'`
        if [ "$tmp_arg" != "$1" ]; then
          $my_echo >&2 "The value of $2 must be a number: $1"
          usage
        fi
      fi
    fi
  fi 
}

decode () {
  printf "%s\n" "$1" | sed  -e 's/[\\]\?!/\&#33;/g' -e 's/\\/\\\\/g' -e 's/"/\&#34;/g' -e "s/'/\&#039;/g" -e 's/</\&#60;/g' -e 's/>/\&#62;/g' -e 's/\&quot;/\&#34;/g' -e 's/\&lt;/\&#60;/g' -e 's/\&gt;/\&#62;/g' -e 's/\&amp;/\&/g' -e 's/\&/\&amp;/g' -e 's/\&amp;#/\&#/g' -e 's/\&#34;/\&quot;/g' -e 's/\&#60;/\&lt;/g' -e 's/\&#62;/\&gt;/g'
}

decode_json () {
  printf "%s\n" "$1" | sed 's/"/\\\\\\"/g'
}

get_joc_option () {
	if [ -r "$SCHEDULER_DATA/config/private/private.conf" ]
	then
		egrep "^$1" "$SCHEDULER_DATA/config/private/private.conf" | cut -d = -f 2 | sed 's/^\s*"*//' | sed 's/"*\s*$//'
	else
		echo ""
	fi
}

parse_events() {
  param1="$1"
  param2="$2"
  param3=`echo "$3" | sed 's/\"//g'`
  shift
  shift
  shift
  if [ "$param1" = "joc" ]; then
    add_or_remove_with_joc2 "again " "$param2" "$param3" "$*" >/dev/null
  else
    add_or_remove_with_curl "again " "$param2" "$param3" "$*" >/dev/null
  fi
  return $?
}

dequeue_events() {
  if [ -f "$1" ]; then
    mv "$1" "$1~"
	num_of_events=`wc -l "$1~" | awk '{print $1}'`
	num_of_suc_events=0
	if [ "$num_of_events" != "0" ]; then
	  $my_echo "trying to dequeue $num_of_events event(s)."
      while read -r line
      do
        if parse_events $line
		then
			num_of_suc_events=`expr $num_of_suc_events + 1`
		fi
      done < "$1~"
	  $my_echo "dequeueing for $num_of_suc_events event(s) successful."
	fi
	rm -f "$1~" >/dev/null 2>/dev/null
  fi
}

queue_events() {
  $my_echo >&2 "Could not signal event to $3 for $4. Event data are stored $1for later dequeueing in ${queued_events}."
  $my_echo "$2" "$3" "\"$4\"" "$5">>"${queued_events}"
}

get_access_token () {
	if [ "$ACCESS_TOKEN" != "" ]; then
	  return 0
	fi
	if [ "$joc_account" = "" ]; then
	  $my_echo >&2 "JOC account is undefined"
	  return 1
	fi
	if [ "$joc_url" = "" ]; then
	  $my_echo >&2 "JOC url is undefined"
	  return 1
	fi
	ANSWER=`curl -k -s -S -u "$joc_account" -X POST -i -m $http_timeout -H "Accept: application/json" -H "Content-Type: application/json" $joc_url/joc/api/security/login`
	if [ $? -eq 0 ]
	then
		HTTP_STATUS=`echo "$ANSWER" | grep "HTTP/"`
		HTTP_CODE=`echo "$HTTP_STATUS" | awk '{print $2}'`
		case "$HTTP_CODE" in
			200)		ACCESS_TOKEN=`echo "$ANSWER" | grep "X-Access-Token:" | sed 's/\x0d//g' | awk '{print $2}'`
						# remove \r (x0d) very important, otherwise you get later :'bad HTTP parsed: 400 Bad EOL for HttpChannelOverHttp'
						return 0
						;;
			420)		$my_echo >&2 "$HTTP_CODE Login failed for user $joc_account";;
			401|403)	$my_echo >&2 "$HTTP_CODE The user $joc_account is not authorized to access JOC Cockpit";;
			*)			$my_echo >&2 "$HTTP_STATUS";;
		esac
	else 
		echo >&2 "$ANSWER"
	fi
	return 1 
}

add_or_remove_with_joc() {
  retcode=1
  if get_access_token
  then
    dequeue_events "$queued_events"
	if add_or_remove_with_joc2 "" "$1" "$2" "$3"
	then 
		retcode=0
	fi
  else 
    queue_events "" "joc" "$1" "$2" "$3"
  fi
  return $retcode
}

add_or_remove_with_joc2() {
  retcode=1
  if get_access_token
  then
	curl -k -s -S -X POST -i -d "{\"jobschedulerId\":\"$3\",\"orders\":[$4]}" -m $http_timeout -H "X-Access-Token: $ACCESS_TOKEN" -H "Accept: text/plain" -H "Content-Type: application/json" $joc_url/joc/api/events/custom/$2 > "${tmpprefix}_answer_jobscheduler_event.tmp"
	if [ $? -eq 0 ]
	then
		HTTP_STATUS=`cat "${tmpprefix}_answer_jobscheduler_event.tmp" | grep "HTTP/"`
		HTTP_CODE=`echo "$HTTP_STATUS" | awk '{print $2}'`
		case "$HTTP_CODE" in
			200)		tail -n 1 "${tmpprefix}_answer_jobscheduler_event.tmp"
						$my_echo ""
						retcode=0
						;;
			420)		tail -n 1 "${tmpprefix}_answer_jobscheduler_event.tmp" >&2
						$my_echo >&2 ""
						;;
			403)		$my_echo >&2 "$HTTP_CODE Forbidden! The user $joc_account doesn't have sufficient permissions";;
			*)			$my_echo >&2 "$HTTP_STATUS";;
		esac
	else 
		cat "${tmpprefix}_answer_jobscheduler_event.tmp" >&2
	fi
	rm -f "${tmpprefix}_answer_jobscheduler_event.tmp" >/dev/null 2>/dev/null
	if [ "$retcode" = "1" ]; then
		queue_events "$1" "joc" "$2" "$3" "$4"
	fi
  else 
    queue_events "$1" "joc" "$2" "$3" "$4"
  fi
  return $retcode
}

check_with_joc () {
  retcode=1
  if get_access_token
  then
    curl -k -s -S -X POST -i -d "{\"jobschedulerId\":\"$scheduler_id\",\"eventClass\":\"$event_class\",\"eventId\":\"$event_id\",\"exitCode\":\"${exit_code2}\",\"xPath\":\"$xpath_expr\"}" -m $http_timeout -H "X-Access-Token: $ACCESS_TOKEN" -H "Accept: text/plain" -H "Content-Type: application/json" $joc_url/joc/api/events/custom/check > "${tmpprefix}_answer_jobscheduler_event.tmp"
	if [ $? -eq 0 ]
	then
		HTTP_STATUS=`cat "${tmpprefix}_answer_jobscheduler_event.tmp" | grep "HTTP/"`
		HTTP_CODE=`echo "$HTTP_STATUS" | awk '{print $2}'`
		case "$HTTP_CODE" in
			200)		tail -n 1 "${tmpprefix}_answer_jobscheduler_event.tmp"
						$my_echo ""
						retcode=0
						;;
			420)		tail -n 1 "${tmpprefix}_answer_jobscheduler_event.tmp" >&2
						$my_echo >&2 ""
						;;
			403)		$my_echo >&2 "$HTTP_CODE Forbidden! The user $joc_account doesn't have sufficient permissions";;
			*)			$my_echo >&2 "$HTTP_STATUS";;
		esac
	else 
		cat "${tmpprefix}_answer_jobscheduler_event.tmp" >&2
	fi
	rm -f "${tmpprefix}_answer_jobscheduler_event.tmp" >/dev/null 2>/dev/null
  fi
  return $retcode
}

add_or_remove_with_curl() {
	retcode=1
	if [ "$1" = "" ]; then
		dequeue_events "$queued_events"
	fi
	curl -k -s -S -X POST -i -d "$4" -m $http_timeout -H "Accept: application/xml" -H "Content-Type: application/xml" "http://$3" > "${tmpprefix}_answer_jobscheduler_event.tmp"
	if [ $? -eq 0 ]
	then
		HTTP_STATUS=`cat "${tmpprefix}_answer_jobscheduler_event.tmp" | grep "HTTP/"`
		HTTP_CODE=`echo "$HTTP_STATUS" | awk '{print $2}'`
		case "$HTTP_CODE" in
			200)		scheduler_response=`cat "${tmpprefix}_answer_jobscheduler_event.tmp" | tr "\n" " " | egrep '<spooler>[^<]*<answer[^>]*>[^<]*<ERROR' | sed 's/.*<spooler>[^<]*<answer[^>]*>[^<]*<ERROR[^>]*text="\([^"]*\)".*/\1/'`
						echo "$scheduler_response"
						if [ "$scheduler_response" = "" ]; then
							$my_echo "JobScheduler response: OK"
							retcode=0
						else
							$my_echo >&2 "JobScheduler reports error: ${scheduler_response}"
						fi
						;;
			400)		$my_echo >&2 "JobScheduler reports error:"
						tail -n 1 "${tmpprefix}_answer_jobscheduler_event.tmp" >&2
						$my_echo >&2 ""
						;;
			*)			$my_echo >&2 "$HTTP_STATUS";;
		esac
	else 
		cat "${tmpprefix}_answer_jobscheduler_event.tmp" >&2
	fi
	rm -f "${tmpprefix}_answer_jobscheduler_event.tmp" >/dev/null 2>/dev/null
	if [ "$retcode" = "1" ]; then
		queue_events "$1" "supervisor" "$2" "$3" "$4"
	fi
	return $retcode
}

check_with_curl() {
	retcode=1
	curl -k -s -S -X POST -i -d "<param.get name=\"JobSchedulerEventJob.events\"/>" -m $http_timeout -H "Accept: application/xml" -H "Content-Type: application/xml" "http://${scheduler_supervisor_host}:${scheduler_supervisor_port}" > "${tmpprefix}_answer_jobscheduler_event.tmp"
	if [ $? -eq 0 ]
	then
		HTTP_STATUS=`cat "${tmpprefix}_answer_jobscheduler_event.tmp" | grep "HTTP/"`
		HTTP_CODE=`echo "$HTTP_STATUS" | awk '{print $2}'`
		case "$HTTP_CODE" in
			200)		scheduler_response=`cat "${tmpprefix}_answer_jobscheduler_event.tmp" | tr "\n" " " | egrep '<spooler>[^<]*<answer[^>]*>[^<]*<ERROR' | sed 's/.*<spooler>[^<]*<answer[^>]*>[^<]*<ERROR[^>]*text="\([^"]*\)".*/\1/'`
						if [ "$scheduler_response" = "" ]; then
							if check_jobscheduler_answer "${tmpprefix}_answer_jobscheduler_event.tmp"
							then
								retcode=0
							fi
						else
							$my_echo >&2 "Job Scheduler reports error: ${scheduler_response}"
						fi
						;;
			400)		$my_echo >&2 "Job Scheduler reports error:"
						tail -n 1 "${tmpprefix}_answer_jobscheduler_event.tmp" >&2
						$my_echo >&2 ""
						;;
			*)			$my_echo >&2 "$HTTP_STATUS";;
		esac
	else 
		cat "${tmpprefix}_answer_jobscheduler_event.tmp" >&2
	fi
	rm -f "${tmpprefix}_answer_jobscheduler_event.tmp" >/dev/null 2>/dev/null
	return $retcode
}

check_jobscheduler_answer () {
    java -classpath "${thisdir}/../lib/sos/*:${thisdir}/../lib/3rd-party/*" sos.scheduler.misc.JobSchedulerEventXPath "$1" "$xpath_expr" "$event_class" "$event_id" "$exit_code2"
    return $?
}

logout () {
	if [ "$ACCESS_TOKEN" = "" ]; then
	  return 0
	fi
	curl -k -s -S -X POST -m $http_timeout -H "Accept: application/json" -H "X-Access-Token: $ACCESS_TOKEN" $joc_url/joc/api/security/logout >/dev/null 2>/dev/null
	ACCESS_TOKEN=""
	return 0
}

if [ $# -eq 0 -o "$1" = "-?" -o "$1" = "/?" -o "$1" = "--help" ]
then
  usage
fi

while getopts x:e:i:b:l:u:n:j:c:o:h:p:s:r:k:w:m:t:a:d: opt
do
    case "$opt" in
      x)  if [ "$what" = "check" ]; then
            check_arg "$OPTARG" "exit code (-x)"
          else
            check_arg "$OPTARG" "exit code (-x)" number
            exit_code="$OPTARG"
          fi
          exit_code2="$OPTARG"
          ;;
      e)  check_arg "$OPTARG" "event class (-e)"
          event_class="$OPTARG"
          ;;
      i)  check_arg "$OPTARG" "event id (-i)"
          event_id="$OPTARG"
          ;;
      b)  check_arg "$OPTARG" "JobScheduler id (-b)"
          scheduler_id="$OPTARG"
          ;;
      u)  check_arg "$OPTARG" "JOC Cockpit url (-u)"
          joc_url="$OPTARG"
          ;;
      n)  check_arg "$OPTARG" "JOC Cockpit account (-n)"
          joc_account="$OPTARG"
          ;;
      j)  check_arg "$OPTARG" "job name (-j)"
          job_name="$OPTARG"
          ;;
      c)  check_arg "$OPTARG" "job chain (-c)"
          job_chain="$OPTARG"
          ;;
      o)  check_arg "$OPTARG" "order id (-o)"
          order_id="$OPTARG"
          ;;
      h)  check_arg "$OPTARG" "workload job scheduler host (-h)"
          scheduler_host="$OPTARG"
          ;;
      p)  check_arg "$OPTARG" "workload job scheduler port (-p)" number
          scheduler_tcp_port="$OPTARG"
          ;;
      s)  check_arg "$OPTARG" "supervisor job scheduler host (-s)"
          scheduler_supervisor_host="$OPTARG"
          ;;
      r)  check_arg "$OPTARG" "supervisor job scheduler port (-r)" number
          scheduler_supervisor_port="$OPTARG"
          ;;
      k)  check_arg "$OPTARG" "supervisor job chain (-k)"
          event_job_chain="$OPTARG"
          ;;
      w)  if [ "${OPTARG}" != "add" -a "${OPTARG}" != "remove" -a "${OPTARG}" != "check" ]; then
            $my_echo >&2 "Invalid action value has been specified: -w add|remove|check"
            usage
          fi
          what="$OPTARG"
          ;;
      m)  check_arg "$OPTARG" "HTTP Timeout in s" number
          http_timeout="$OPTARG"
          ;;
      t)  if [ "$expire_param" != "" ]; then
            $my_echo >&2 "expiration date (-t), expiration cycle (-tc) and expiration period (-tp) may not be used at the same time."
          fi
          case "$OPTARG" in
            "p" ) shift `expr $OPTIND - 1`
                  tmp_var=`$my_echo "$1" | sed -e 's/^[0-9]\{2\}\(:[0-9]\{2\}\)\{1,2\}//'` 
                  if [ "$tmp_var" != "" ]; then
                    $my_echo >&2 "Invalid expiration period value has been specified: -tc (hh:mm[:ss])"
                    usage
                  fi
                  expire_param="<param name='expiration_period' value='$1'/>"
                  expire_json_param="{\"name\":\"expiration_period\",\"value\":\"$1\"},"
                  expires="expiration_period=$1"
                  shift
                  OPTIND=1
                  ;;
            "c" ) shift `expr $OPTIND - 1`
                  tmp_var=`$my_echo "$1" | sed -e 's/^[0-9]\{2\}\(:[0-9]\{2\}\)\{1,2\}//'` 
                  if [ "$tmp_var" != "" ]; then
                    $my_echo >&2 "Invalid expiration cycle value has been specified: -tc (hh:mm[:ss])"
                    usage
                  fi
                  expire_param="<param name='expiration_cycle' value='$1'/>"
                  expire_json_param="{\"name\":\"expiration_cycle\",\"value\":\"$1\"},"
                  expires="expiration_cycle=$1"
                  shift
                  OPTIND=1
                  ;;
            "d" ) shift `expr $OPTIND - 1`
                  tmp_var=`$my_echo "$1" | sed -e 's/T/ /'  -e 's/^[0-9]\{4\}-[0-9]\{2\}-[0-9]\{2\} [0-9]\{2\}:[0-9]\{2\}:[0-9]\{2\}//'` 
                  if [ "$1" != "never" -a "$tmp_var" != "" ]; then
                    $my_echo >&2 "Invalid expiration date value has been specified: -td never|(ISO date)"
                    usage
                  fi
                  expire_param="<param name='expires' value='$1'/>"
                  expire_json_param="{\"name\":\"expires\",\"value\":\"$1\"},"
                  expires="expires=$1"
                  shift
                  OPTIND=1
                  ;;
            *   ) tmp_var=`$my_echo "$OPTARG" | sed -e 's/T/ /'  -e 's/^[0-9]\{4\}-[0-9]\{2\}-[0-9]\{2\} [0-9]\{2\}:[0-9]\{2\}:[0-9]\{2\}//'` 
                  if [ "$OPTARG" != "never" -a "$tmp_var" != "" ]; then
                    $my_echo >&2 "Invalid expiration date value has been specified: -t never|(ISO date)"
                    usage
                  fi
                  expire_param="<param name='expires' value='$OPTARG'/>"
                  expire_json_param="{\"name\":\"expires\",\"value\":\"$OPTARG\"},"
                  expires="expires=$OPTARG"
                  ;;
          esac            
          ;;
      a)  check_arg "$OPTARG" "xpath expression (-a)" expression
          xpath_expr="$OPTARG"
          ;;
      l)  allowed_exit_codes="$OPTARG"
          ;;
      d)  pname=`printf "%s\n" "$OPTARG" | cut -d = -f 1 -s`
          if [ "$pname" = "" ]; then
            printf "%s\n" "Missing equal sign in \"$OPTARG\"" >&2
            usage
          fi
          check_arg "$pname" "name in -d"
          pvalue=`printf "%s\n" "$OPTARG" | cut -d = -f 2- -s`
          pvalue=`decode "$pvalue"`
          json_pvalue=`decode_json "$pvalue"`
          parameters="${parameters}<param name='${pname}' value='${pvalue}'/>"
          json_parameters="${json_parameters}{\"name\":\"${pname}\",\"value\":\"${json_pvalue}\"},"
          ;;
      \?) # unknown flag 
          usage
          ;;
    esac
done

shift `expr $OPTIND - 1`

if [ "$what" != "check" -a "${event_class}" = "" ]; then
  $my_echo >&2 "Parameter missing: -e event-class"
  usage
fi

#if [ "$scheduler_id" = "" ]; then
#  $my_echo >&2 "JobScheduler id (-I) is required (-s)"
#  usage
#fi

if [ "$joc_url" = "" ]; then
  joc_url=`get_joc_option "joc\.url"`
fi

#if [ "$joc_url" = "" ]; then
#  $my_echo >&2 "JOC Cockpit url id is required (-J)"
#  usage
#fi

if [ "$joc_account" = "" ]; then
  joc_account=`get_joc_option "joc\.account"`
fi

#if [ "$joc_account" = "" ]; then
#  $my_echo >&2 "JOC Cockpit account (user:[password]) is required (-A)"
#  usage
#fi

if [ "$joc_url" != "" -a "$joc_account" != "" -a "$scheduler_id" != "" ]; then
  use_web_service=1
fi

if [ "$http_timeout" = "" ]; then
  http_timeout=`get_joc_option "http\.timeout"`
fi

if [ "$http_timeout" = "" ]; then
  http_timeout=15000
fi
http_timeout=`expr ${http_timeout} / 1000`

test "$scheduler_supervisor_host" = "" && scheduler_supervisor_host=${scheduler_host}
test "$scheduler_supervisor_port" = "" && scheduler_supervisor_port=${scheduler_tcp_port}

if [ "$scheduler_supervisor_host" = "" -o "$scheduler_supervisor_port" = "" ]; then
  if [ "$use_web_service" = "0" ]; then
    $my_echo >&2 "JobScheduler Supervisor or JOC Cockpit parameters have not been specified: \n" \
              "-s JobScheduler host -r JobScheduler port\n" \
              "or\n" \
              "-J JOC Cockpit url -A JOC Cockpit account -I JobScheduler Id"
    usage
  fi
fi

test "$what" = "check" && exit_code=0
test "${exit_code}" = "" && exit_code=0
if [ "${event_id}" = "" -a "$what" = "add" ]; then
  event_id=${exit_code}
fi

# check exit code for allowed values
for allowed_exit_code in `echo "$allowed_exit_codes" | sed 's/,/ /g'`
do
    check_arg "$allowed_exit_code" "allowed exit codes" number
    if [ "$allowed_exit_code" = "${exit_code}" ]; then
      # Job terminated successfully: exit code has allowed value $1
      exit_code=0
    fi
done

test "$what" = "add" && exit_code2=${exit_code}

# add environment variables starting with SCHEDULER_PARAM_ to parameters
for scheduler_param in `set | egrep "^SCHEDULER_PARAM_" | sed -e 's/^SCHEDULER_PARAM_//'`
do
  pname=`$my_echo "$scheduler_param" | cut -d = -f 1 -s | tr A-Z a-z`
  test "$pname" = "" && continue
  test "$pname" = "jobschedulereventjob.events" && continue
  pvalue=`$my_echo "$scheduler_param" | cut -d = -f 2- -s` 
  pvalue=`decode "$pvalue"`
  json_pvalue=`decode_json "$pvalue"`
  parameters="${parameters}<param name='${pname}' value='${pvalue}'/>"
  json_parameters="${json_parameters}{\"name\":\"${pname}\",\"value\":\"${json_pvalue}\"},"
done

# submit event to Supervisor JobScheduler instance
errorlevel=0
scheduler_response_found=0

# submit event to Supervisor JobScheduler instance via JOC
errorlevel=0
if [ "$what" = "check" ]; then
  if [ "$use_web_service" = "1" ]; then
    check_with_joc
  else 
    check_with_curl
  fi
else
  if [ "$use_web_service" = "1" ]; then
    add_or_remove_with_joc "$what" "${scheduler_id}" \
    "{\"jobChain\":\"${event_job_chain}\",\"params\":[${expire_json_param}${json_parameters}{\"name\":\"scheduler_id\",\"value\":\"${scheduler_id}\"},{\"name\":\"job_chain\",\"value\":\"${job_chain}\"},{\"name\":\"order_id\",\"value\":\"${order_id}\"},{\"name\":\"job_name\",\"value\":\"${job_name}\"},{\"name\":\"event_class\",\"value\":\"${event_class}\"},{\"name\":\"event_id\",\"value\":\"${event_id}\"},{\"name\":\"exit_code\",\"value\":\"${exit_code2}\"}]}"
  else 
    add_or_remove_with_curl "" "$what" "${scheduler_supervisor_host}:${scheduler_supervisor_port}" \
    "<add_order job_chain='${event_job_chain}'><params><param name='action' value='${what}'/><param name='remote_scheduler_host' value='${scheduler_host}'/><param name='remote_scheduler_port' value='${scheduler_tcp_port}'/><param name='job_chain' value='${job_chain}'/><param name='order_id' value='${order_id}'/><param name='job_name' value='${job_name}'/><param name='event_class' value='${event_class}'/><param name='event_id' value='${event_id}'/><param name='exit_code' value='${exit_code2}'/><param name='created' value='${now}'/>${expire_param}${parameters}</params></add_order>"
  fi
fi
errorlevel=$?
logout

if [ "$errorlevel" != "0" ]; then
  if [ "$what" = "check" ]; then
    exit_code="$errorlevel"
  else
    test -z "${exit_code2}" || exit_code="${exit_code2}" 
  fi
fi

exit $exit_code